// CC-004 - Happy Moon Fish
//
// For the love of god, don't code a real game like this, okay?
//
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "raylib.h"


#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

#define SCREEN_W          1000
#define SCREEN_H          1000
#define GLSL_VERSION       330

#define MAX_ACTORS          64
#define MAX_TEXTBOX         16
#define MAX_ANIMATIONS      16
#define MAX_FRAMES          16
#define MAX_SFX             32

#define PURE_GREEN       (Color){   0, 255,   0, 255 }
#define PURE_BLUE        (Color){   0,   0, 255, 255 }
#define PURE_YELLOW      (Color){ 255, 255,   0, 255 }
#define SCORE_PINK       (Color){ 242, 103, 119, 255 }
#define SCORE_ORANGE     (Color){ 255, 161,   0, 255 }
#define WATER            (Color){  25, 113, 180, 255 }
#define SCORE_GREEN      (Color){ 145, 240,   0, 255 }

#define SKY_BLUE         (Color){  25, 113, 180, 255 }

#define MOOD_BLUE        (Color){  20, 200, 250, 255 }
#define MOOD_GREEN       (Color){  30, 250,  30, 255 }
#define MOOD_YELLOW      (Color){ 250, 215,  30, 255 }
#define MOOD_RED         (Color){ 255,  50,  50, 255 }
#define PLAYER_COLOR     (Color){ 205, 180, 145, 255 }


#define SESSION_TIME     10800
#define MOVE_SCALE        4.00
#define TILE_SIZE        16.00
#define ACTOR_SCALE       4.00

#define PUMPKIN_H_SPEED   1.50
#define PUMPKIN_H_ARC     3.50

#define PUMPKIN_L_SPEED   2.00
#define PUMPKIN_L_ARC     2.00

#define GRAVITY_PULL      0.10
#define CENTER_X           500
#define CENTER_Y           500






// Update routines (should be called per-frame, or at fixed intervals)
// Project-specific routines are listed last
//
void check_textbox();
void check_animations();
void check_time();
void check_score();

void check_actors();
void check_recharge();

void cleanup();
void end_game();
void show_stats();

// Spawning and actor maintenance routines
//


void kill_actor(int slot);
int  spawn_actor(int type, float x, float y, float z, int cell_x, float path_x, float path_y, float path_z);
int  spawn_fish();

void assign_animation(int who, int anim, int loops);
void kill_animation(int who);



// Drawing routines
//
void draw_actor(int slot);
void draw_summary();
void draw_tutorial();
void draw_high_score();
void draw_title_hud();
void draw_hud();



void draw_text_box(int slot);
void trigger_textbox(int slot, int x, int y, int w, int h, int message, int delay, int duration);

void update_and_render();
void render_frame(int mode);

void init_game_variables();


int  get_free_actor();
int  get_population(int team);
int  get_initial();

Vector2 get_sprite_projection(int who);                    // Projects actor into the world
int check_floats(float one, float two, float range);       // Checks if two floats are within [range]
int check_move_collision(int who, float x, float z);       // Checks if actor's coords impinge on another's





// Old Framework stuff here
int sorted[MAX_ACTORS][2];

void FR_merge_actors();
void FR_prep_for_sort();
void FR_quicksort(int low, int high);
void FR_swap_int(int n1, int n2, int index);
int  FR_2d_dist(int x1, int y1, int x2, int y2);
// End nonsense


// Wrapping nonsense
void blit(Image src, Image dest, int s_x, int s_y, int d_x, int d_y, int w, int h);
void blit_v(Image src, Image dest, int s_x, int s_y, int d_x, int d_y, int w, int h);
void blit_h(Image src, Image dest, int s_x, int s_y, int d_x, int d_y, int w, int h);
void push_rect(Rectangle *rect, float x, float y, float w, float h);



char custom_text[80];

const int screenWidth  = 1000;
const int screenHeight = 1000;

double rad_conv = (PI / 180);

int calculate     = 0;  // Defers calculations for so many frames
int tutorial_seen = 0;
int game_over     = 0;  // Triggers end of game (0 = game active. 1 = summary. 2 = high score)
int total_actors  = 0;  // Debugging var

int population[2] = { 0, 0 };

int select        = 0;  // Current player's menu selection
int accept        = 0;  // Current player's prompt selection
int yes_no        = 0;  // shows prompt when > 0

int score         = 0;  // Your score
int score_diff    = 0;  // Score differential. Positive values add, negative subtract.
int score_shake   = 0;  // Number of frames to "shake" score display. Used for negative hits.
int score_derps   = 0;  // Keeps track of consecutive misses
int feeds         = 0;
int fubars        = 0;

int high_score    = 666;
unsigned char high_init[4] = { 71, 68, 82, 0 };

int slow_time     = 0;  // > 0 if time is slowed by player
int slow_frame    = 0;  // Current number of updates skipped by slowdown

int lurve_tex_x   = 0;  // For anchoring happy texture at game over summary
int lurve_tex_y   = 0;  //
int lurve_update  = 0;  //

int water_tex_x   = 0;  // For anchoring happy texture at game over summary
int water_tex_y   = 0;  //
int water_update  = 0;  //

int clock_min     = 0;  // Countdown readout
int clock_sec     = 0;  //
int clock_sub     = 0;  //

int quake_ticks   = 1800; // Frames before next quake
int quake_base    = 3600; // Base ticks
int quake_spread  = 1800; // Randomness to quakes
int quake_duration = 0;  // Quake frames left to animation

int game_ticks    = SESSION_TIME; // Number of clock ticks left in game (60 = 1 second)
int score_ticks   = 0;            // User for score tracking
int frame_ticks   = 0;            // Used for end game animations
int puppet        = -1;           // Actor being controlled by the player

int kill_off_step[4] = { -1, -1, -1, -1 };

int game_mode     = 0; // 0 = title.    1 = Tutorial.     2 = game.    3 = Game Over.     4 = Credits?
float anims[16]   = { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
float t_angle[4]  = { 0.0, 0.0, 0.0, 0.0 };
float t_offset[4] = { 0.0, 90.0, 180.0, 270.0 };
int warning[16]   = { 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0 };


int p_rate        = 60;  // Maximum ticks between ammo recharges
int p_max         = 10;  // Max ammo count
int p_light       = 0;   // Number of light shots player has fired
int p_heavy       = 0;   // Heavy shots fired
int p_l_hit       = 0;   // Light shots that landed on-target
int p_h_hit       = 0;   // Heavy shots

Camera2D camera      = { 0 };
Vector2 cam_velocity = { 0, 0 };

Vector2 mouse      = { 0, 0 };
Vector2 mouse_cam  = { 0, 0 };

Texture2D tex_SHIPS[8]; // Each ship on-screen has to get its own unique texture
                        // We can't update fast enough to re-use a single texture

Texture2D tex_stars;
Texture2D tex_moon;
Texture2D tex_title;
Texture2D tex_64x;
Texture2D lurve_tex;

Sound sfx[MAX_SFX];

Font font1;         // Bubble font
Font font2;         // Deco font
Font font3;         // 3x5 font



struct textbox_struct
{
 Rectangle pos;
 float scale;
 bool enabled;                // TRUE if this box is active
 int viewable_text;           // Number of chars currently exposed (starts at zero, counts up)
 int duration;                // Number of ticks before textbox closes. -1 if disabled
 int mode;                    // 0 = entering, 1 = displaying text, 2 = exiting
 int ticks;                   // Counts time between displaying characters
 const char *msg;
};

textbox_struct textbox[MAX_TEXTBOX];





struct anim_struct
{
 int frame_x[MAX_FRAMES];     // x cell of frame inside sprite sheet (cell, not pixel)
 int frame_y[MAX_FRAMES];     // y cell of frame inside sprite sheet
 int timing[MAX_FRAMES];      // timing in game "ticks" to get to next frame
 int total_frames;            // maximum number of frames in this sequence
};

anim_struct anim[MAX_ANIMATIONS];





struct actor_struct
{
 Texture2D cel;         // Pointer to source texture
 Vector3 pos;           // Position
 Vector3 offset;        // Destination (only used to move enemies around)
 Vector3 dest;          // Destination (only used to move enemies around)
 Vector3 path;          // Path of travel
 Color color;           // Color tint (WHITE by default)

 float blend;           // Blend strength (0-1)    < only applies if not solid >
 float angle;           // Angle of rotation (0-360)
 float rotation;        // amount to rotate each frame
 float scale;           // Actor scale (populated during projection)
 float roll;            // != 0 if actor rolls while in flight (typically +/- 0.5)
 float y_shift;         // Dirty hack to physically shift a sprite on y axis by ? pixels (pre-projection)

 float quake;           // > 0 if actor is rolling from damage
 float sinking;         // > 0 if actor is sinking into the sand
 float speed;           // Actor's base speed (0.0 to 0.20) which is added to their movement each turn

 int team;              // 1 = Fremen   2 = Harkonnen
 int _size_w;           // Dimensions of the cel frame
 int _size_h;           // Dimensions of the cel frame
 int flip;              // Force horizontal mirroring
 int anim_set;          // Animation set to reference
 int anim_step;         // Which step in the anim sequence we're at
 int anim_timing;       // Current step timing for animation
 int anim_loop;         // # of times to loop animation. 0 if only played once
 int mode;              // 0 = solid. 1 = alpha
 int state;             // 0 = inactive. 1-12 = doing stuff
 int effect;            // 0 = none.  1 = zap-glow
 int home;              // Actor they are targeting (worm to thumper)
 int timer;             // Internal timer for animation purposes.
 int stamina;           // Walking and being attacked drain stamina. When <= 0 the actor dies or fades out
 int cel_x;             // X cel within sprite sheet
 int cel_y;             // Y cel within sprite sheet
 int locked_to;         // Actor this player was triggered by (so ammo doesn't hit the actor firing it!)
 int committed;         // > 0 if actor is committed to an activity
 int event;             // > 0 if actor will trigger an event when their commitment has ended

 float scale_ramp;      // Maximum value scale additive will ramp to
 float scale_mod;       // Modifies scale variable, when rendering sprite. This is a factor.
 float scale_up;        // Speed of scale ramping
 float scale_down;      // Speed of scale ramping

 int ship;              // > -1 if actor is the owner of a ship package
 int crew;              // Crew left
 int last_fired;        // Last cannon that fired (0-3)
 int ammo;              // Ammo left
 int candy;             // Candy in ship's hold
 int hull;              // Actor associated with the hull of the ship
 int oars[8];           // Actors associated with oars
 int captain;           // Actor associated with the captain
 int cape;              // Actor associated with the captain's cape
 int p_ticks;           // Current Ticks between ammo recharges
 int moved_up;          // > 0 if actor has moved up recently
 int moved_down;        // > 0 if actor has moved down recently
 int moved_wait;        // > 0 if actor needs to wait before heading to next way-point

 int d_pad[4];          // Janky virtual gamepad (refine later)
 int buttons[4];        //

 int shader;            // > 0 if a shader is in use with this actor
 float s_vars[10];      // Storage space for shader vars. These will vary according to the shader in use

 bool decal;            // Indicates actor is a floor decal, and will be drawn before normal actors
 bool fighter;          // Quick way of assigning actor as fighter
 bool flicker;          // Tracks actor flicker (used by effects)
 bool mirror;           // TRUE if actor is mirrored on X-axis
 bool shadow;           // TRUE if this actor has a shadow
 bool gravity;          // TRUE if gravity affects Y coordinate
 bool projected;        // TRUE is actor is passed through projection stage before drawing
 bool enabled;          // TRUE if this actor is active
 bool accessory;        // TRUE if actor will be rendered with a package object
                        // and should not be drawn or interacted with otherwise
};

actor_struct actors[MAX_ACTORS];





// I got tired of making everything global. Fuck me.
//
struct hi_struct
{
 int selected;                // Character currently selected (0-2 valid)
 unsigned char initials[3];   // Stores characters entered by user
 int blink;                   // True if we can display blinking chars
 int blink_timer;             // Timer to track above variable
};

hi_struct hi_score;






int main(void)
{
 int i    = 0;
 int k    = 0;
 int next = 0;
 int exit = 0;
 char rep_str[37];

 float scale = 0.0;



    // Init OpenGL, etc
    //
    //SetConfigFlags(FLAG_WINDOW_UNDECORATED);
    InitWindow(screenWidth, screenHeight, "CC-004 - Happy Moon Fish");
    InitAudioDevice();

    SetTargetFPS(60);
    HideCursor();

    // Load textures
    //
    tex_title        = LoadTexture("gfx/tex_title.png");
    tex_moon         = LoadTexture("gfx/tex_moon.png");
    tex_64x          = LoadTexture("gfx/tex_64x.png");
    tex_stars        = LoadTexture("gfx/tex_stars.png");
    lurve_tex        = LoadTexture("gfx/lurve_tex.png");

    // Custom font loading
    //
    font1 = LoadFont("gfx/16x_bubble.png");
    font2 = LoadFont("gfx/16x_deco.png");
    font3 = LoadFont("gfx/3x_square.png");


    // Load sound effects
    //
    sfx[2]  = LoadSound("sfx/002.wav"); // OK GUI beep
    sfx[3]  = LoadSound("sfx/003.wav"); // Wrong/NOPE GUI beep
    sfx[5]  = LoadSound("sfx/005.wav"); // Long-range shot
    sfx[6]  = LoadSound("sfx/006.wav"); // Successful Feeding
    sfx[7]  = LoadSound("sfx/007.wav"); // Total fuck-up



    // Set sound effect attributes
    //
    SetSoundVolume(sfx[2], 0.25);
    SetSoundPitch(sfx[2],  1.00);

    SetSoundVolume(sfx[3], 0.25);
    SetSoundPitch(sfx[3],  1.00);

    SetSoundVolume(sfx[5], 0.50);
    SetSoundPitch(sfx[5],  1.00);

    SetSoundVolume(sfx[6], 0.25);
    SetSoundPitch(sfx[6],  1.00);

    SetSoundVolume(sfx[7], 0.50);
    SetSoundPitch(sfx[7],  1.00);


    // We will call this routine to clean up on program exit
    //
    atexit(cleanup);


    // We will land here after a successful game loop
    //
    restart:


    init_game_variables();


    /// Title block loop ---------------------------------------------------------------------------------------
    //
    do {
        anims[0] = 1.00;
        update_and_render();

///        anims[15] += 0.05;
///        if(anims[15] >= 360.00) { anims[15] -= 360.00; }

        if(IsKeyDown(KEY_Z)) { next = 1; }
        if(next && (!IsKeyDown(KEY_Z))) { PlaySound(sfx[2]); exit = 1; }

        // Spawn player, then populate the moon with fish
        //
        if(puppet == -1) {
            k = spawn_actor(0, 500, 500, 0, 1, 0.0, 0.0, 0.0);
            actors[k].team    = 0;
            actors[k].angle   = 0.0;
            actors[k].scale   = 2.0;
            actors[k].pos.z   = 330.0;
            actors[k].cel     = tex_64x;
            actors[k]._size_w = 64;
            actors[k]._size_h = 64;
            actors[k].color   = PLAYER_COLOR;
            actors[k].enabled = 1;
            puppet = k;

            for(i=0; i<14; i++) { spawn_fish(); }
                         }


       } while (!exit);



    // Fade Title out
    //
    do {
        anims[0] -= 0.025;
        update_and_render();
       } while (anims[0] > 0.00);



    /// Tutorial Loop ------------------------------------------------------------------------------------------
    //
    if(!tutorial_seen) {
        // Tutorial Display loop (only on first game of each session)
        //
        next      = 0;
        exit      = 0;
        game_mode = 1;

        do {
            update_and_render();

            if(IsKeyDown(KEY_Z)) { next = 1; }
            if(next && (!IsKeyDown(KEY_Z))) { PlaySound(sfx[2]); exit = 1; }

           } while (!exit);


        // Fade Title out
        //
        do {
            anims[1] -= 0.04;
            update_and_render();
           } while (anims[1] > 0.00);

        tutorial_seen = 1;
                       }



    /// Main Game Loop -----------------------------------------------------------------------------------------
    //
    next      = 0;
    exit      = 0;
    game_mode = 2;

    while (!game_over) {

        update_and_render();
                       }


    // End game summary and high score
    //
    end_game();

    goto restart;

    return(0);
}





// This routine is automatically called at program exit
//
void cleanup()
{
 int i = 0;
 float rotation = 0.0;

 // Free resources and clean up
 //
 UnloadTexture(tex_stars);
 UnloadTexture(tex_title);
 UnloadTexture(tex_moon);
 UnloadTexture(tex_64x);
 UnloadTexture(lurve_tex);

 UnloadFont(font1);
 UnloadFont(font2);
 UnloadFont(font3);

 UnloadSound(sfx[2]);
 UnloadSound(sfx[3]);
 UnloadSound(sfx[5]);
 UnloadSound(sfx[6]);
 UnloadSound(sfx[7]);

 // Probably need to unload images too lol
 //

 CloseWindow();                // Close window and OpenGL context

 return;
}






// Performs logic update and calls rendering stuff
//
void update_and_render()
{
 int moved      = 0;
 int stride     = 0;
 int i          = 0;
 float rotation = 0.0;
 float offset   = 0.0;

 total_actors   = 0;


 if(WindowShouldClose()) { exit(0); }


 // Floating object animation
 //
 anims[2] += 2.0;
 if(anims[2] > 360.00) { anims[2] -= 360.00; }


 // Flashing mini-map indicator animation
 //
 anims[3] += 0.05;
 if(anims[3] > 1.00) { anims[3] -= 1.00; }


 // UI selection pulsing animation
 //
 anims[4] += 4.0;
 if(anims[4] > 360.00) { anims[4] -= 360.00; }


 // Quake damage pulsing (fed into sin())
 //
 anims[5] += 8.0;
 if(anims[5] > 360.00) { anims[5] -= 360.00; }


 // Holding position shuffle (slow, lazy sine)
 //
 anims[6] += 0.01;
 if(anims[6] > 360.00) { anims[6] -= 360.00; }


 // Floating animation (for lack of a true behavior)
 //
 if(puppet != -1) {

     if(IsKeyDown(KEY_LEFT))  { moved = 1; actors[puppet].mirror = 0; rotation =  0.20; offset = 0.003485;  }
     if(IsKeyDown(KEY_RIGHT)) { moved = 1; actors[puppet].mirror = 1; rotation = -0.20; offset = -0.003485; }

     if(rotation != 0.0) {
         anims[7] += rotation;
         if(anims[7] > 360.00) { anims[7] -= 360.00; }

         for(i=0; i<MAX_ACTORS; i++) {
             if(actors[i].enabled && i != puppet && actors[i].team != 2) { actors[i].angle += offset; }
                                     }
                         }

     // Let player "bounce" as they move
     //
     if(moved) {
         stride = sin(anims[4]) * 6.00;
         if(stride > 0) { actors[puppet].pos.z = 330 + stride; }
               }
     else { actors[puppet].pos.z = 330; }


 // Update positions
 //if(puppet != -1) { actors[puppet].angle += 0.003485; }
                  }



 // Cape waving animation (fed into sin())
 //
 anims[8] += 8.0;
 if(anims[8] > 360.00) { anims[8] -= 360.00; }


 // Captain waving animation
 //
 anims[9] += 0.05;
 if(anims[9] >= 2.00) { anims[9] -= 2.00; }


 // Stars wander in the background
 //
 anims[15] += 0.10;
 if(anims[15] >= 360.00) { anims[15] -= 360.00; }

 check_textbox();
 check_actors();


 if(!game_over && game_mode > 1) {
     check_animations();
     check_recharge();
     check_score();

     if(!slow_time || (slow_time && slow_frame > 3)) {
         check_time();
         slow_frame = 0;
                                                     }
     else { slow_frame++; }
                }

 render_frame(game_mode);

 return;
}







void render_frame(int mode)
{
 int i   = 0;
 int j   = 0;
 int k   = 0;
 int t_x = 0;
 int t_y = 0;

 float mouse_scale = 1.0;
 float offset      = 0.0;
 float angle       = 0.0;
 float y_scale     = 0.5;
 float scale       = 1.0;
 float x_shift     = 0.0;

 Rectangle tex_s;
 Rectangle tex_d;

 int line_offset[12] = { 0, 54, 171, 23,  98, 0, 161, 220,  0, 54, 171, 23 }; // Offset layers so they don't align visibly


 camera.target = (Vector2){ 0,0 };
 camera.offset = (Vector2){ 0,0 };


 BeginDrawing();

 DrawRectangle(0, 0, 1000, 1000, BLACK);

 DrawTexture(tex_stars, -300 + (sin(anims[15] * rad_conv) * 200), -300 + (cos(anims[15] * rad_conv) * 200), WHITE);

 DrawTexturePro(tex_moon, (Rectangle){0,0,1000,1000}, (Rectangle){500,500,1000,1000}, (Vector2){500,500}, anims[7], WHITE);


 // Sort actors
 //
 FR_prep_for_sort();
 FR_quicksort(0,(MAX_ACTORS-1));
 FR_merge_actors();


 /// sprintf(custom_text, "FPS = %d", GetFPS());
 /// DrawTextEx(font1, custom_text, (Vector2){10, screenHeight - 24}, font1.baseSize * 1, 1, WHITE);


 // Having drawn all the necessary stuff, handle mode-specific overlays
 //
 switch(game_mode) {
     case 0: // Title screen
             draw_title_hud();
             break;

     case 1: // Introduction / Tutorial
             draw_tutorial();
             break;

     case 3: // Game over / Summary
             draw_summary();
             break;
                   }


 if(!game_over && game_mode > 1) { draw_hud(); }

 EndDrawing();


 return;
}





// NOTE: Typical usage is to put the box over a character or tile's position
//
void draw_text_box(int slot)
{
 Rectangle dest;
 char strang[80];

 Vector2 text_coords;
 Vector2 text_len;


 if(!textbox[slot].enabled) { return; }
 if(textbox[slot].mode < 0) { return; }


 // Calculate textbox dimensions, etc
 //
 dest.width  = textbox[slot].pos.width  * textbox[slot].scale;
 dest.height = textbox[slot].pos.height * textbox[slot].scale;

 dest.x = textbox[slot].pos.x - (dest.width / 2.0f);
 dest.y = textbox[slot].pos.y - (dest.height / 2.0f);


 // Draw border around box
 //
 DrawRectangleLinesEx(dest, 4, WHITE);


 // Draw tiled texture (happy face) and update anchor coordinates
 //
 Rectangle tex_s = { lurve_tex_x,lurve_tex_y, 64, 32 };
 Rectangle tex_d = { dest.x+3, dest.y+3, dest.width - 6, dest.height - 6 };
 DrawTextureTiled(lurve_tex, tex_s, tex_d, (Vector2){0,0}, 0.0, 3.0, Fade(WHITE, 0.90));


 // Measure text and box and figure out how to center text
 //
 text_len = MeasureTextEx(font2, textbox[slot].msg, (font2.baseSize * 2), 1);

 text_coords.x = dest.x + (dest.width / 2) - (text_len.x / 2);
 text_coords.y = dest.y + (dest.height / 2.0f) - font2.baseSize;


 // Draw characters
 //
 if(textbox[slot].viewable_text > 0) {
     memset(strang, 0, sizeof(strang));
     strncpy(strang, textbox[slot].msg, textbox[slot].viewable_text);
     DrawTextEx(font2, strang, text_coords, (font2.baseSize * 2), 1, WHITE);
                                     }

 return;
}





void draw_high_score()
{

 return;
}





void check_textbox()
{
 int i = 0;
 int len = 0;

 for(i=0; i<MAX_TEXTBOX; i++) {
     if(textbox[i].enabled) {
         switch(textbox[i].mode) {
                     break;

             case 0: // Entering
                     textbox[i].scale += 0.075;
                     if(textbox[i].scale >= 1.00) { textbox[i].scale = 1.00; textbox[i].mode++; }
                     break;

             case 1: // Displaying text
                     len = strlen(textbox[i].msg);
                     if(textbox[i].viewable_text < len) {
                         textbox[i].ticks++;
                         if(textbox[i].ticks > 3) { textbox[i].viewable_text++; PlaySound(sfx[5]); textbox[i].ticks = 0; }
                                                        }
                     else { textbox[i].ticks++; } // Keep counting ticks, in case useful

                     if(textbox[i].duration != -1) {
                         if(textbox[i].ticks > textbox[i].duration) { textbox[i].mode = 2; }
                                                   }
                     break;

             case 2: // Exiting
                     textbox[i].viewable_text = 0;
                     textbox[i].scale -= 0.075;
                     if(textbox[i].scale <= 0.00) { textbox[i].scale = 0.00; textbox[i].enabled = 0; }
                     break;
                                 }

         if(textbox[i].mode < 0) { textbox[i].mode++; } // Pause before beginning, effectively

                            }
                              }

 // Update background texture position
 //
 if(!lurve_update) {
     lurve_tex_x--;
     lurve_tex_x &= 0x3F;

     lurve_tex_y++;
     lurve_tex_y &= 0x1F;

     lurve_update = 1;
                   }

 else { lurve_update = 0; }


 // Update water texture position
 //
 if(water_update > 60) {
     water_tex_x++;
     water_tex_x &= 0xFF;

     water_tex_y = 0;
     water_update = 0;
                   }

 else { water_update++; }

 return;
}






// Adjusts score over time
//
void check_score()
{
 int factor = 1;
 int total  = 0;
 int i      = 0;

 score_ticks++;
 if(score_ticks >= 15) {

     // Tabulate score per game tick
     //
     for(i=0; i<MAX_ACTORS; i++) {
         if(actors[i].enabled && actors[i].team == 1) {
             if(actors[i].stamina <=  0) { score_diff +=   1; } // Red
             if(actors[i].stamina >   0) { score_diff +=   4; } // Yellow
             if(actors[i].stamina > 300) { score_diff +=  10; } // Blue
             if(actors[i].stamina > 700) { score_diff +=  25; } // Green
                                                      }
                                 }

     score_ticks -= 15;
                       }


 // Score shakes when bad things happen
 //
 if(score_shake > 0) { score_shake--; }


 // Roll differential into score over time. The larger the difference,
 // the faster we will update the score.
 //
 if(score_diff != 0) {

     total = abs(score_diff);
     if(total > 20)   { factor = 1;  }
     if(total > 50)   { factor = 2;  }
     if(total > 100)  { factor = 4;  }
     if(total > 250)  { factor = 12; }
     if(total > 500)  { factor = 16; }
     if(total > 999)  { factor = 24; }
     if(total > 1999) { factor = 32; }
     if(total > 3999) { factor = 48; }
     if(total > 6999) { factor = 64; }

     if(score_diff > 0) { score += factor; score_diff -= factor; }
     if(score_diff < 0) { score -= factor; score_diff += factor; }
                     }

 // Bounds check
 //
 if(score < 0)        { score = 0;        }
 if(score > 99999999) { score = 99999999; }


 // Map score into this project's destination
 //
 //actors[puppet].candy = score;

 return;
}






void check_animations()
{
 int i = 0;
 int SET = 0;

 for(i=0; i<MAX_ACTORS; i++) {
     if(actors[i].enabled && actors[i].anim_set) {
         SET = actors[i].anim_set;

         actors[i].anim_timing++;

         /// printf("%d,",actors[i].anim_timing);

         // Check for frame advance
         //
         if(actors[i].anim_timing >= anim[SET].timing[actors[i].anim_step]) {
             actors[i].anim_step++;
             actors[i].anim_timing = 0;

             // Check for end, or looping
             //
             if(actors[i].anim_step >= anim[SET].total_frames) {
                 if(actors[i].anim_loop) {
                     actors[i].anim_step = 0;
                     actors[i].anim_timing = 0;
                     actors[i].anim_loop--;
                                         }

                 else { actors[i].anim_set = 0; }
                                                               }
                                                                            }

         // If anim is still set, update frame
         // ** SHOULD PLACE IN ROUTINE SO WE CAN SANITY CHECK ANIM_STEP VALUE **
         //
         if(actors[i].anim_set) {
             SET = actors[i].anim_set;
             actors[i].cel_x = anim[SET].frame_x[actors[i].anim_step];
             actors[i].cel_y = anim[SET].frame_y[actors[i].anim_step];
                                }
                                                 }
                             }

 return;
}





// NOTE: Keeps track of game time, and parses the ticks left
//       into human readable format
//
void check_time()
{
 div_t d;
 div_t e;

 if(game_mode != 2) { return; }

 d = div(game_ticks, 3600);

 clock_min = d.quot;

 e = div(game_ticks - (d.quot * 3600), 60);
 clock_sec = e.quot;
 clock_sub = e.rem;


 // If time is up, or we are out of fish, stop the game
 //
 if(clock_min <= 0 && clock_sec <= 0 && clock_sub <= 0) { game_over = 1; return; }
 if(get_population(1) <= 0) { game_over = 1; return; }


 // If below 15 seconds left, sound warning
 //
 if(clock_min <= 0 && clock_sec < 16 && clock_sec >= 0) {
     if(!warning[clock_sec]) {
         PlaySound(sfx[3]);
         warning[clock_sec] = 1;
                             }
                                                        }

 game_ticks--;


 return;
}







// Periodically spawns a new mood enhancer emoji
//
void check_recharge()
{
 int i = 0;
 int k = 0;

 i = GetRandomValue(0, 1000);
 if(i >= 960) {

     if(i > 995) { k = spawn_actor(0, 500, 500, 0, 2, 0.0, 0.0, 0.0); }
     else        { k = spawn_actor(0, 500, 500, 0, 3, 0.0, 0.0, 0.0); }

     if(k != -1) {
         actors[k].state   = 2;
         actors[k].team    = 2;
         actors[k].angle   = (float)GetRandomValue(0, 6200) / 1000.0;
         actors[k].pos.z   = 450.0 + GetRandomValue(0, 200);
         actors[k].cel     = tex_64x;
         actors[k]._size_w = 64;
         actors[k]._size_h = 64;
         actors[k].stamina = 1000;
         actors[k].color   = WHITE;
         actors[k].enabled = 1;
                 }
              }


 return;
}









// Kills off dead actors, spawns new ones over time
//
void check_actors()
{
 int i        = 0;
 int j        = 0;
 int k        = 0;

 int prey     = 0;
 int effect   = 0;
 float volume = 1.0;
 float dist   = 0.0;
 float speed  = 0.1;
 Vector3 coords = { 0, 0 };


 for(i=0; i<MAX_ACTORS; i++) {
     if(actors[i].enabled) {

         if(game_mode == 2) {
             if(actors[i].team == 1) {
                 actors[i].stamina--;
                 if(actors[i].stamina < -2500) { actors[i].state = 4; }
                                     }
                            }

         // Color based off of mood (stamina)
         // Goodies are immune to this behavior (team = 2)
         //
         if(i != puppet && actors[i].team != 2) {
             if(actors[i].stamina < 1)   { actors[i].color = MOOD_RED; }
             if(actors[i].stamina > 700) { actors[i].color = MOOD_GREEN; }
             if(actors[i].stamina > 300 && actors[i].stamina <= 700) { actors[i].color = MOOD_BLUE; }
             if(actors[i].stamina > 0 && actors[i].stamina <= 300) { actors[i].color = MOOD_YELLOW; }
                                                }

         if(actors[i].quake > 0.000) {
             actors[i].angle = sin(anims[5]) * actors[i].quake;
             actors[i].quake -= 0.15;
             if(actors[i].quake <= 0.000) {
                 actors[i].angle = 0.0;
                 actors[i].quake = 0.0;
                                          }
                                     }


         switch(actors[i].state) {
             case 0: // Inactive
                     break;
/*
             case 1: // Moving (somewhere in general)
                     actors[i].pos.x += actors[i].path.x;

                     if(actors[i].gravity) { actors[i].path.y -= GRAVITY_PULL; actors[i].angle += actors[i].roll; }

                     actors[i].pos.z += (actors[i].path.z * actors[i].scale);

                     actors[i].pos.y += actors[i].path.y;
                     if(actors[i].pos.y < -0.1) {

                         if(actors[i].stamina == 666) { // crew
                             effect = 10;
                             volume = 0.50 - (actors[i].pos.z * 0.0015);
                             if(volume > 0.75) { volume = 0.75; }
                             SetSoundVolume(sfx[effect], volume);
                             SetSoundPitch(sfx[effect],  0.80 + (float)GetRandomValue(0, 200) / 1000.00);
                             PlaySound(sfx[effect]);
                                                      }

                         else { // sand
                             effect = 9;

                             volume = 0.75 - (actors[i].pos.z * 0.0015);
                             if(volume > 0.75) { volume = 0.75; }
                             SetSoundVolume(sfx[effect], volume);
                             SetSoundPitch(sfx[effect],  0.80 + (float)GetRandomValue(0, 200) / 1000.00);
                             PlaySound(sfx[effect]);
                              }

                         spawn_splash(i);
                         kill_actor(i);
                         break;
                                                }
                     break;
*/
             case 1: // Moving along path (tracks impact)
                     actors[i].pos.x += ((actors[i].path.x) * actors[i].speed);

                     if(actors[i].gravity) { actors[i].path.y -= GRAVITY_PULL; actors[i].angle += actors[i].roll; }

                     actors[i].pos.z += (actors[i].path.z * actors[i].scale);
                     actors[i].pos.y += actors[i].path.y;

                     break;

             case 2: // Floating down to the moon
                     actors[i].pos.z -= (0.50 + actors[i].sinking);
                     actors[i].sinking += 0.007;
                     if(actors[i].pos.z < 285.00) { actors[i].state = 4; }
                     break;

             case 3: // Floating (Candy only, if you need something else to float use another state)
                     actors[i].pos.y += sin(actors[i].angle + anims[7]) * 0.3;
                     actors[i].stamina--;

                     if(actors[i].stamina <= 0) { actors[i].state = 4; }
                     break;

             case 4: // Rising, rotating, expiring
                     actors[i].pos.z += 0.25;
                     actors[i].blend -= 0.04;
                     if(actors[i].blend <= 0.00 ) { kill_actor(i); }
                     break;

                                 }
                           }
                             }

 return;
}












void kill_actor(int slot)
{

 // Basic Sanity Check
 //
 if(slot < 0 || slot >= MAX_ACTORS) { return; }

 int i = 0;

 actors[slot].enabled     = 0;
 actors[slot].projected   = 0;
 actors[slot].mirror      = 0;
 actors[slot].team        = 0;
 actors[slot].decal       = 0;
 actors[slot].shader      = 0;
 actors[slot].shadow      = 0;
 actors[slot].fighter     = 0;
 actors[slot].accessory   = 0;
 actors[slot].y_shift     = 0;
 actors[slot].gravity     = 0;
 actors[slot].flicker     = 0;
 actors[slot].anim_set    = 0;
 actors[slot].anim_step   = 0;
 actors[slot].anim_timing = 0;
 actors[slot].anim_loop   = 0;
 actors[slot].effect      = 0;
 actors[slot].state       = 0;
 actors[slot].ship        = -1;
 actors[slot].mode        = 0;
 actors[slot].flip        = 0;
 actors[slot].timer       = 0;
 actors[slot]._size_w     = 16; // Default value
 actors[slot]._size_h     = 16; // Default value
 actors[slot].home        = -1;
 actors[slot].committed   = 0;
 actors[slot].event       = 0;
 actors[slot].locked_to   = -1;
 actors[slot].scale       = 1.0;
 actors[slot].rotation    = 0.0;
 actors[slot].blend       = 1.0f;
 actors[slot].angle       = 0.0f;
 actors[slot].roll        = 0.0f;
 actors[slot].quake       = 0.0f;
 actors[slot].sinking     = 0.0f;
 actors[slot].speed       = 1.0f; // Default value
 actors[slot].stamina     = 1000;

 actors[slot].scale_ramp  = 0.0;
 actors[slot].scale_mod   = 1.0;
 actors[slot].scale_up    = 0.0;
 actors[slot].scale_down  = 0.0;

 actors[slot].crew        = 0;
 actors[slot].candy       = 0;
 actors[slot].last_fired  = 0;
 actors[slot].p_ticks     = 0;
 actors[slot].ammo        = 0;
 actors[slot].hull        = -1;
 actors[slot].captain     = -1;
 actors[slot].moved_up    = 0;
 actors[slot].moved_down  = 0;
 actors[slot].moved_wait  = 0;

 for(i=0; i<6; i++) { actors[slot].oars[i] = -1; }
 actors[slot].oars[6] = 0;
 actors[slot].oars[7] = 0;

 for(i=0; i<4; i++) { actors[slot].d_pad[i] = 0; actors[slot].buttons[i] = 0; }

 for(i=0; i<10; i++) { actors[slot].s_vars[i] = 0.0; }

 actors[slot].path.x      = 0;
 actors[slot].path.y      = 0;
 actors[slot].path.z      = 0;

 actors[slot].pos.x       = 0;
 actors[slot].pos.y       = 0;
 actors[slot].pos.z       = 0;

 actors[slot].dest.x      = 0;
 actors[slot].dest.y      = 0;
 actors[slot].dest.z      = 0;

 actors[slot].offset.x    = 0;
 actors[slot].offset.y    = 0;
 actors[slot].offset.z    = 0;

 actors[slot].cel         = tex_64x;
 actors[slot].color       = WHITE;
 return;
}






// Spawns a new actor. Types are 0-x for default skin
//
int spawn_actor(int type, float x, float y, float z, int cell_x, float path_x, float path_y, float path_z)
{
 int i    = 0;
 int slot = get_free_actor();

 // Basic Sanity Check
 //
 if(slot < 0 || slot >= MAX_ACTORS) { return(-1); }

 actors[slot].mode      = 0;
 actors[slot].effect    = 0;
 actors[slot].projected = 1;
 actors[slot].blend     = 1.0f;
 actors[slot].angle     = 0.0f;
 actors[slot].timer     = 0;
 actors[slot].stamina   = GetRandomValue(500, 1200); // was 350, 500
 actors[slot].rotation  = 0.0;
 actors[slot].locked_to = -1;

 actors[slot].pos.x     = x;
 actors[slot].pos.y     = y;
 actors[slot].pos.z     = z;

 actors[slot].path.x    = path_x;// * ((float)GetRandomValue(50, 200) / 100.0);
 actors[slot].path.y    = path_y;// * (float)GetRandomValue(1, 2);
 actors[slot].path.z    = path_z;// * (float)GetRandomValue(1, 2);

 actors[slot].cel       = tex_64x; // default value
 actors[slot].cel_x     = cell_x;
 actors[slot].cel_y     = 0;

 actors[slot].state     = 1;
 actors[slot].scale     = 1.0;

 actors[slot].enabled = 1;

 return(slot);
}







// Returns a slot to a free actor.
// If something goes wrong, we will clobber actor 0.
// Do something more graceful later
//
int get_free_actor()
{
 int i = 0;

 for(i=1; i<MAX_ACTORS; i++) {
     if(!actors[i].enabled) { return(i); }
                             }

 return(0);
}






void draw_actor(int slot)
{
 // Basic Sanity Check
 //
 if(slot < 0 || slot >= MAX_ACTORS) { return; }

 if(!actors[slot].enabled) { return; }

 float x_flip    = 1.0;
 float s_flip    = 0.0;
 float m_scale   = 0.0;
 float dist      = 0.0;
 Vector2 coords  = { actors[slot].pos.x, actors[slot].pos.y };
 Vector2 coords2 = { 0, 0 };
 Vector2 origin  = { actors[slot]._size_w / 2, actors[slot]._size_h / 2 };

 total_actors++;

 // Flip sprite horizontally if path of travel is negative on x axis
 //
 if(actors[slot].flip) { x_flip = -1.0; s_flip = - 4; }

 Rectangle source  = { (actors[slot]._size_w * actors[slot].cel_x), (actors[slot]._size_h * actors[slot].cel_y), actors[slot]._size_w * x_flip, actors[slot]._size_h };

 if(actors[slot].sinking) {
     source.height -= actors[slot].sinking;
                          }


 if(actors[slot].projected) { coords = get_sprite_projection(slot); }

 m_scale = actors[slot].scale;

 Rectangle dest     = { coords.x - ((actors[slot]._size_w * (m_scale-1.0)) / 2), coords.y - ((actors[slot]._size_h * (m_scale -1.0)) / 2), actors[slot]._size_w * m_scale, actors[slot]._size_h * m_scale };
 Rectangle dest_alt = { 0, 0, 0, 0 };


 if(actors[slot].mirror) { source.width *= -1; }

 switch(actors[slot].effect) {
     case 2: // Shot has a menacing aura (flickers)
             if(actors[slot].flicker) {
                 //DrawTexturePro(tex_aura, source_alt, dest_alt, origin_alt, actors[slot].angle * 2.0, Fade(actors[slot].color, actors[slot].blend));
                 actors[slot].flicker = 0;
                                      }
             else { actors[slot].flicker = 1; }

             DrawTexturePro(actors[slot].cel, source, dest, origin, actors[slot].angle, Fade(actors[slot].color, actors[slot].blend));
             break;

    default: // No effects
             // Shadow first, then sprite
             //if(actors[slot].shadow && actors[slot].pos.y < 0.2) {
             //    DrawTexturePro(tex_sprites, (Rectangle){32, 0, 16, 16}, dest, origin, actors[slot].angle, Fade(actors[slot].color, actors[slot].blend));
             //                                                    }
             DrawTexturePro(actors[slot].cel, source, dest, origin, actors[slot].angle * 57.0, Fade(actors[slot].color, actors[slot].blend));
             break;
                             }


 // Check for collision (we could optimize by building a set of fish, but... slacking!)
 //
 if(actors[slot].pos.z < 300.00 && actors[slot].state == 2) {
     for(int k=0; k<MAX_ACTORS; k++) {
         if(actors[k].enabled && actors[k].team == 1 && slot != k) {
             coords2 = get_sprite_projection(k);
             dist    = FR_2d_dist(coords.x + 32, coords.y + 32, coords2.x + 32, coords2.y + 32);
             if(dist < 32) {
                 actors[slot].state = 4;

                 // If "good" emoji, fill stamina to max level
                 // If "bad" emoji, kill off fish and deduct from score
                 //
                 if(actors[slot].cel_x == 3) { PlaySound(sfx[6]); actors[k].stamina = 1000; feeds++; }
                 else                        { PlaySound(sfx[7]); actors[k].state =4; score_diff -= 7500; fubars++; }

                 k = 1000;
                           }
                                                                   }
                                     }
                                                            }

 return;
}







int get_population(int team)
{
 int i = 0;
 int total = 0;

 for(i=1; i<MAX_ACTORS; i++) {
     if(actors[i].enabled) {
         if(actors[i].team == team) { total++; }
                           }
                             }

 return(total);
}






// NOTE: Draws the top/bottom banners for the title screen
//       Should only have minor changes between projects
//
void draw_title_hud()
{
 Vector2 pos = { 0, 0 };
 Vector2 len = { 0, 0 };

 DrawTexture(tex_title, 0, 600, WHITE);


 len   = MeasureTextEx(font1, "PRESS [Z] TO START", font1.baseSize, 1);
 pos.x = (screenWidth / 2) - (len.x / 2);
 pos.y = screenHeight - 24;

 sprintf(custom_text, "PRESS [Z] TO START");
 DrawTextEx(font1, custom_text, pos, font1.baseSize, 1, Fade(WHITE, (1.0 * anims[3])));

 return;
}







void draw_hud()
{


 DrawTextEx(font1, "- TIME -", (Vector2){ 84, 12 }, font1.baseSize, 1, YELLOW);
 sprintf(custom_text, "%02d:%02d:%02d", clock_min, clock_sec, clock_sub);
 DrawTextEx(font1, custom_text, (Vector2){ 20, 28 }, font1.baseSize * 2, 1, WHITE);


 DrawTextEx(font1, "- FISH -", (Vector2){ 436, 12 }, font1.baseSize, 1, YELLOW);
 sprintf(custom_text, "%02d", get_population(1));
 DrawTextEx(font1, custom_text, (Vector2){ 470, 28 }, font1.baseSize * 2, 1, WHITE);


 DrawTextEx(font1, "- SCORE -", (Vector2){ 776, 12 }, font1.baseSize, 1, YELLOW);
 sprintf(custom_text, "%08d", score);
 DrawTextEx(font1, custom_text, (Vector2){ 720, 28 }, font1.baseSize * 2, 1, WHITE);

/* // Draw Control Hints
 //
 DrawTextEx(font3, "Z - HEAVY SHOT", (Vector2){10, screenHeight - 45}, font3.baseSize * 2, 1, Fade(WHITE, 0.5));
 DrawTextEx(font3, "X - LIGHT SHOT", (Vector2){10, screenHeight - 30}, font3.baseSize * 2, 1, Fade(WHITE, 0.5));
 DrawTextEx(font3, "ARROWS - MOVE", (Vector2){10, screenHeight - 15}, font3.baseSize * 2, 1, Fade(WHITE, 0.5));
*/
 return;
}







// Copied-over framework stuff goes here



void FR_merge_actors()
{
 int i = 0;

 // Decals first
 //
 for(i=0; i<MAX_ACTORS; i++) {
     if(sorted[i][1] != -1) {
         if(actors[sorted[i][1]].decal) { draw_actor(sorted[i][1]); }
                            }
                             }

 // Decals first
 //
 for(i=0; i<MAX_ACTORS; i++) {
     if(sorted[i][1] != -1) {
         if(!actors[sorted[i][1]].decal) { draw_actor(sorted[i][1]); }
                            }
                             }

 return;
}







// Add hook to allow for different manners of sorting outside the framework
//
// NOTE: For use with quicksort implementation
//
void FR_prep_for_sort()
{
 int i    = 0;

     for(i=0; i<MAX_ACTORS; i++) {
         sorted[i][0] = -1;
         sorted[i][1] = -1;

         if(actors[i].enabled && !actors[i].accessory) {
             sorted[i][0] = i; //0 - actors[i].pos.z + (actors[i].angle / 6.20);
             sorted[i][1] = i;
                                                       }
                                 }

 return;
}






// NOTE: OLD CODE IS OLD. Quicksort. Does what it says, says what it does.
//
void FR_quicksort(int low, int high)
{
 int top    = low;
 int bottom = (high - 1);
 int part_index;
 int part_value;

 if (low < high) {

     // Check if elements are sequential
     //
     if(high == (low + 1)) {
         if(sorted[low][0] > sorted[high][0]) { FR_swap_int(high, low, 0); }
                           }

     else {
         // Choose a partition element and swap it with
         // the last value in the array
         //
         part_index = (int)((low + high) / 2);
         part_value = sorted[part_index][0];
         FR_swap_int(high, part_index, 0);

         do {
	         // Inrement the top pointer
             //
	         while((sorted[top][0] <= part_value) && (top <= bottom)) { top++; }

	         // Decrement the bottom pointer
             //
	         while((sorted[bottom][0] > part_value) && (top <= bottom)) { bottom--; }

             // Swap elements if pointers have not met
             //
	         if(top < bottom) { FR_swap_int(top, bottom, 0); }
            } while(top < bottom);

     FR_swap_int(top, high, 0);

     // Recursive calls
     //
     FR_quicksort(low, top - 1);
     FR_quicksort(top + 1, high);
          }

                 }

 return;
}






// NOTE: OLD CODE IS OLD. For use with quicksort implementation
//
void FR_swap_int(int n1, int n2, int index)
{
 int temp;
 int temp2;

 temp  = sorted[n1][0];
 temp2 = sorted[n1][1];

 sorted[n1][0] = sorted[n2][0];
 sorted[n1][1] = sorted[n2][1];

 sorted[n2][0] = temp;
 sorted[n2][1] = temp2;

 return;
}







// OLD CODE IS OLD.
//
int FR_2d_dist(int x1, int y1, int x2, int y2)
{
 double d_x, d_y;
 double distance;

 d_x = x1 - x2;
 d_y = y1 - y2;

 d_x *= d_x;
 d_y *= d_y;

 distance = sqrt((d_x + d_y));

 return((int)distance);
}










void init_game_variables()
{
 int i = 0;
 int j = 0;

 int feeds = 0;
 int fubars = 0;


 /// Old stuff

 score       = 0;
 score_diff  = 0;
 score_shake = 0;
 score_ticks = 0;
 slow_time   = 0;
 slow_frame  = 0;
 yes_no      = 0;
 select      = 0;
 accept      = 0;
 p_light     = 0;
 p_heavy     = 0;
 p_l_hit     = 0;
 p_h_hit     = 0;
 puppet      = -1;

 for(i=0; i<16; i++) { warning[i] = 0; }
 for(i=0; i<16; i++) { anims[i] = 1.0; }
 for(i=0; i<4; i++)  { kill_off_step[i] = -1; }

 for(i=0; i<MAX_TEXTBOX; i++) {
     textbox[i].pos.x = 0.0;
     textbox[i].pos.y = 0.0;
     textbox[i].pos.width  = 0;
     textbox[i].pos.height = 0;
     textbox[i].scale = 0.0;
     textbox[i].mode = 0;
     textbox[i].ticks = 0;
     textbox[i].viewable_text = 0;
     textbox[i].msg = NULL;
     textbox[i].enabled = 0;
                              }


 // Define camera
 //
 camera.target = (Vector2){ 4096, 4096 };
 camera.offset = (Vector2){ 0, 0 };
 camera.rotation = 0.0f;
 camera.zoom = 1.0f;


 // Set up actors and spawns
 //
 for(i=0; i<MAX_ACTORS; i++) { kill_actor(i); }

 //generate_spawns();

 game_over     = 0;
 lurve_tex_x   = 0;
 lurve_tex_y   = 0;
 lurve_update  = 0;
 game_ticks    = SESSION_TIME;
 frame_ticks   = 0;
 game_mode     = 0;


/*
 if(puppet == -1) {
     puppet = spawn_ship(0, 700, 0, 400);
     actors[puppet].crew = 100;
     actors[puppet].color = WHITE;
     //actors[actors[puppet].cape].color = BLUE;
                  }
*/

 return;
}









// Ends game upon time-out.
// Player gets a review of their stats
// Player is then booted to the title screen (vars need to be reset)
//
void end_game()
{
 int exit = 0;
 int key  = 0;

 game_mode = 3;

 do {
     update_and_render();
    } while(!IsKeyDown(KEY_Z));

 PlaySound(sfx[2]);

 do {
     update_and_render();
    } while(IsKeyDown(KEY_Z));


 return;
}






// Out of laziness, check each key and send back anything unique
// Valid keys are A-Z and 0-9. Returns 0 if no button seen
//
int get_initial()
{
 int check = 0;

 if(IsKeyPressed(KEY_A)) { return(65); }

 if(IsKeyPressed(KEY_B))  { return(66); }
 if(IsKeyPressed(KEY_C))  { return(67); }
 if(IsKeyPressed(KEY_D))  { return(68); }
 if(IsKeyPressed(KEY_E))  { return(69); }
 if(IsKeyPressed(KEY_F))  { return(70); }
 if(IsKeyPressed(KEY_G))  { return(71); }

 if(IsKeyPressed(KEY_H))  { return(72); }
 if(IsKeyPressed(KEY_I))  { return(73); }
 if(IsKeyPressed(KEY_J))  { return(74); }
 if(IsKeyPressed(KEY_K))  { return(75); }
 if(IsKeyPressed(KEY_L))  { return(76); }
 if(IsKeyPressed(KEY_M))  { return(77); }
 if(IsKeyPressed(KEY_N))  { return(78); }
 if(IsKeyPressed(KEY_O))  { return(79); }
 if(IsKeyPressed(KEY_P))  { return(80); }

 if(IsKeyPressed(KEY_Q))  { return(81); }
 if(IsKeyPressed(KEY_R))  { return(82); }
 if(IsKeyPressed(KEY_S))  { return(83); }
 if(IsKeyPressed(KEY_T))  { return(84); }
 if(IsKeyPressed(KEY_U))  { return(85); }
 if(IsKeyPressed(KEY_V))  { return(86); }
 if(IsKeyPressed(KEY_W))  { return(87); }
 if(IsKeyPressed(KEY_X))  { return(88); }
 if(IsKeyPressed(KEY_Y))  { return(89); }
 if(IsKeyPressed(KEY_Z))  { return(90); }

 if(IsKeyPressed(KEY_ZERO))  { return(48); }
 if(IsKeyPressed(KEY_ONE))   { return(49); }
 if(IsKeyPressed(KEY_TWO))   { return(50); }
 if(IsKeyPressed(KEY_THREE)) { return(51); }
 if(IsKeyPressed(KEY_FOUR))  { return(52); }
 if(IsKeyPressed(KEY_FIVE))  { return(53); }
 if(IsKeyPressed(KEY_SIX))   { return(54); }
 if(IsKeyPressed(KEY_SEVEN)) { return(55); }
 if(IsKeyPressed(KEY_EIGHT)) { return(56); }
 if(IsKeyPressed(KEY_NINE))  { return(57); }

 return(check);
}










// NOTE: Allegro-style CPU blitting operation
//
void blit(Image src, Image dest, int s_x, int s_y, int d_x, int d_y, int w, int h)
{
 Rectangle s, d;


 s.x = s_x;
 s.y = s_y;
 s.width  = w;
 s.height = h;

 d.x = d_x;
 d.y = d_y;
 d.width  = w;
 d.height = h;

 ImageDraw(&dest, src, s, d, WHITE);

 return;
}






// Vertically-flipped blit.
//
void blit_v(Image src, Image dest, int s_x, int s_y, int d_x, int d_y, int w, int h)
{
 Rectangle s, d;


 s.x = s_x;
 s.y = s_y;
 s.width  = w;
 s.height = h;

 d.x = d_x;
 d.y = d_y;
 d.width  = w;
 d.height = h;

 Image subs = ImageFromImage(src, s);
 ImageFlipVertical(&subs);

 s.x = 0;
 s.y = 0;
 ImageDraw(&dest, subs, s, d, WHITE);

 UnloadImage(subs);

 return;
}




// Horizontally-flipped blit.
//
void blit_h(Image src, Image dest, int s_x, int s_y, int d_x, int d_y, int w, int h)
{
 Rectangle s, d;


 s.x = s_x;
 s.y = s_y;
 s.width  = w;
 s.height = h;

 d.x = d_x;
 d.y = d_y;
 d.width  = w;
 d.height = h;

 Image subs = ImageFromImage(src, s);
 ImageFlipHorizontal(&subs);

 s.x = 0;
 s.y = 0;
 ImageDraw(&dest, subs, s, d, WHITE);

 UnloadImage(subs);

 return;
}




void assign_animation(int who, int anim, int loops)
{
 // Sanity Check
 //
 if(who < 0 || who >= MAX_ACTORS) { return; }

 actors[who].anim_set    = anim;
 actors[who].anim_step   = 0;
 actors[who].anim_timing = 0;
 actors[who].anim_loop   = loops;

 return;
}





void kill_animation(int who)
{
 // Sanity Check
 //
 if(who < 0 || who >= MAX_ACTORS) { return; }

 actors[who].anim_set    = 0;
 actors[who].anim_step   = 0;
 actors[who].anim_timing = 0;
 actors[who].anim_loop   = 0;

 return;
}





void trigger_textbox(int slot, int x, int y, int w, int h, int message, int delay, int duration)
{

 printf("TEXTBOX\n");
 // Sanity check slot and message

 textbox[slot].pos.x      = x;
 textbox[slot].pos.y      = y;
 textbox[slot].pos.width  = w;
 textbox[slot].pos.height = h;
 textbox[slot].scale      = 0.0;
 textbox[slot].duration   = duration;
 textbox[slot].mode       = 0 - delay;
 textbox[slot].ticks      = 0;
 textbox[slot].viewable_text = 0;
 textbox[slot].msg        = NULL; //hints[message];
 textbox[slot].enabled    = 1;

 return;
}





void show_stats()
{
 bool exit = 0;
 bool enter = 1;

 do {

    //actors[puppet].committed = 1000;

    if(enter)  { if(!IsKeyDown(KEY_X)) { enter = 0; } }
    if(!enter) { if(IsKeyDown(KEY_X))  { exit  = 1; } }

    update_and_render();

    } while(!exit);

 //actors[puppet].committed = 0;

 return;
}







// Draws game over score summary
//
void draw_summary()
{
 int txt_x     = 0;
 float pulse   = 0.0;
 float percent = 0.0;
 Vector2 len   = { 0, 0 };

 // Draw lurve texture as background
 //
 Rectangle tex_s = { lurve_tex_x,lurve_tex_y, 64, 32 };
 Rectangle tex_d = { 0, 0, screenWidth, screenHeight };
 DrawTextureTiled(lurve_tex, tex_s, tex_d, (Vector2){0,0}, 0.0, 3.0, Fade(WHITE, 0.90));


 // Game Over Screen Title (duh)
 //
 len   = MeasureTextEx(font1, "GAME  OVER", (font1.baseSize * 4), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);

 DrawTextEx(font1, "GAME  OVER", (Vector2){txt_x, 35}, font1.baseSize * 4, 1, WHITE);


 // Score
 //
 sprintf(custom_text, "TOTAL SCORE      %08d", score);
 DrawTextEx(font1, custom_text, (Vector2){85, 200}, font1.baseSize * 2, 1, SCORE_GREEN);


 // Shot metrics
 //
 if(p_light > 0) { percent = ((float)p_l_hit / (float)p_light) * 100.00; }
 else            { percent = 0.0;                                        }

 sprintf(custom_text, "SUCCESSFUL FEEDINGS %05d", feeds);
 DrawTextEx(font1, custom_text, (Vector2){85, 300}, font1.baseSize * 2, 1, YELLOW);


 if(p_heavy > 0) { percent = ((float)p_h_hit / (float)p_heavy) * 100.00; }
 else            { percent = 0.0;                                        }

 sprintf(custom_text, "FUCK UPS            %05d", fubars);
 DrawTextEx(font1, custom_text, (Vector2){85, 380}, font1.baseSize * 2, 1, YELLOW);




 // Press 'Z' to continue
 //
 len   = MeasureTextEx(font1, "[ PRESS 'Z' TO CONTINUE ]", (font1.baseSize * 2), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);


 pulse = sin(anims[4] * rad_conv) * 0.50;
 DrawTextEx(font1, "[ PRESS 'Z' TO CONTINUE ]", (Vector2){txt_x, screenHeight - 65}, font1.baseSize * 2, 1, Fade(WHITE, 0.50 + pulse));

 return;
}






// Draws ghetto-ass tutorial
//
void draw_tutorial()
{
 int txt_x   = 0;
 float pulse = 0.0;
 Vector2 len = { 0, 0 };

 // Draw lurve texture as background
 //
 Rectangle tex_s = { lurve_tex_x,lurve_tex_y, 64, 32 };
 Rectangle tex_d = { 0, 0, screenWidth, screenHeight };
 DrawTextureTiled(lurve_tex, tex_s, tex_d, (Vector2){0,0}, 0.0, 3.0, Fade(WHITE, 0.90));


 len   = MeasureTextEx(font1, "INSTRUCTIONS", (font1.baseSize * 4), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);

 DrawTextEx(font1, "INSTRUCTIONS", (Vector2){txt_x, 44}, font1.baseSize * 4, 1, WHITE);


 // NOTE: Next project, move this into a routine to clean this janky shit up
 //
 len   = MeasureTextEx(font1, "MOVE THE PLANET LEFT/RIGHT WITH THE ARROW KEYS", (font1.baseSize * 1), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);
 DrawTextEx(font1, "MOVE THE PLANET LEFT/RIGHT WITH THE ARROW KEYS", (Vector2){txt_x, 150}, font1.baseSize * 1, 1, WHITE);


 len   = MeasureTextEx(font1, "MOON FISH FEED ON HAPPY EMOJIS. KEEP THEM FED", (font1.baseSize * 1), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);
 DrawTextEx(font1, "MOON FISH FEED ON HAPPY EMOJIS. KEEP THEM FED", (Vector2){txt_x, 210}, font1.baseSize * 1, 1, WHITE);


 len   = MeasureTextEx(font1, "FISH WILL DIE IF THEY STARVE - OR EAT BAD EMOJIS", (font1.baseSize * 1), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);
 DrawTextEx(font1, "FISH WILL DIE IF THEY STARVE - OR EAT BAD EMOJIS", (Vector2){txt_x, 240}, font1.baseSize * 1, 1, WHITE);


 len   = MeasureTextEx(font1, "WELL FED FISH INCREASE YOUR SCORE", (font1.baseSize * 1), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);
 DrawTextEx(font1, "WELL FED FISH INCREASE YOUR SCORE", (Vector2){txt_x, 270}, font1.baseSize * 1, 1, WHITE);


 len   = MeasureTextEx(font1, "GREEN = GOOD   BLUE = OKAY   YELLOW = POOR    RED = BAD", (font1.baseSize * 1), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);
 DrawTextEx(font1, "GREEN = GOOD   BLUE = OKAY   YELLOW = POOR    RED = BAD", (Vector2){txt_x, 330}, font1.baseSize * 1, 1, WHITE);


 len   = MeasureTextEx(font1, "THE GAME ENDS WHEN TIME IS UP OR FISH ARE DEAD", (font1.baseSize * 1), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);
 DrawTextEx(font1, "THE GAME ENDS WHEN TIME IS UP OR FISH ARE DEAD", (Vector2){txt_x, 360}, font1.baseSize * 1, 1, WHITE);


 len   = MeasureTextEx(font1, "GREEN IS GOOD. RED IS BAD.", (font1.baseSize * 1), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);
 DrawTextEx(font1, "GREEN IS GOOD. RED IS BAD.", (Vector2){txt_x, 470}, font1.baseSize * 1, 1, YELLOW);




 // Press 'Z' to continue
 //
 len   = MeasureTextEx(font1, "[ PRESS 'Z' TO CONTINUE ]", (font1.baseSize * 2), 1);
 txt_x = (screenWidth / 2) - (len.x / 2);

 pulse = sin(anims[4] * rad_conv) * 0.50;
 DrawTextEx(font1, "[ PRESS 'Z' TO CONTINUE ]", (Vector2){txt_x, screenHeight - 65}, font1.baseSize * 2, 1, Fade(WHITE, 0.50 + pulse));

 return;
}










int spawn_fish()
{
 int who = -1;

 who = spawn_actor(0, 500, 500, 0, 0, 0.0, 0.0, 0.0);

 if(who != -1) {
     actors[who].angle   = (float)GetRandomValue(0, 6200) / 1000.0;
     actors[who].team    = 1;
     actors[who].pos.z   = 285.0;
     actors[who].cel     = tex_64x;
     actors[who]._size_w = 64;
     actors[who]._size_h = 64;
     actors[who].stamina = GetRandomValue(750, 1000);
     actors[who].color   = WHITE;
     actors[who].enabled = 1;
               }

 return(who);
}









// NOTE: Checks two floats and returns 1 if they are within range
//
int check_floats(float one, float two, float range)
{

 float check = fabs(one - two);

 if(check <= range) { return(1); }

 return(0);
}






// NOTE: Checks for collision with another ship before moving
//       Pass pre-movement coords in
//       returns 0 if all clear. Returns 1 if there's a collision
//
int check_move_collision(int who, float x, float z)
{
 int i         = 0;
 float check_z = 0.0;
 float check_x = 0.0;

 float X_TOLERANCE = 65.00;
 float Z_TOLERANCE = 30.00;

 for(i=0; i<MAX_ACTORS; i++) {
     if(actors[i].enabled && actors[i].ship != -1) {
         if(i != who) {
             check_x = fabs(x - actors[i].pos.x);
             check_z = fabs(z - actors[i].pos.z);

             if(check_x < X_TOLERANCE && check_z < Z_TOLERANCE) { return(1); }
                      }
                                                   }
                             }

 return(0);
}











void push_rect(Rectangle *rect, float x, float y, float w, float h)
{

 rect->x      = x;
 rect->y      = y;
 rect->width  = w;
 rect->height = h;

 return;
}







/// SHITE MATH BELOW THIS POINT. GO NO FURTHER. SAVE YOUR BRAIN CELLS.



// NOTE: Projects the actor into the world (janky, but whatever)
//       Returns a Vector2 with the final position and dimensions
//
Vector2 get_sprite_projection(int who)
{
 Vector2 coords = { 0, 0 };

 // We assume that [who] is sanitized before calling this

 coords.x = CENTER_X + (sin(actors[who].angle) * actors[who].pos.z);
 coords.y = CENTER_Y - (cos(actors[who].angle) * actors[who].pos.z);


 return(coords);
}









